actor User {}

resource Repository {
  permissions = ["read", "push", "delete"];
  roles = ["contributor", "maintainer", "admin"];

  # A user has the "read" permission if they have the
  # "contributor" role.
  "read" if "contributor";

  # A user has the "push" permission if they have the
  # "maintainer" role.
  "push" if "maintainer";

  # A user has the "delete" permission if they have the
  # "admin" role.
  "delete" if "admin";

  # A user has the "maintainer" role if they have
  # the "admin" role.
  "maintainer" if "admin";

  # A user has the "contributor" role if they have
  # the "maintainer" role.
  "contributor" if "maintainer";
}

# This rule tells Oso how to fetch roles for a repository
has_role(actor: User, role_name: String, repository: Repository) if
  role in actor.roles and
  role_name = role.name and
  repository = role.repository;

has_permission(_actor: User, "read", repository: Repository) if
  repository.is_public;

allow(actor, action, resource) if
  has_permission(actor, action, resource);
